---
title: llarp/dns/srv_data.hpp

---

# llarp/dns/srv_data.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::dns](Namespaces/namespacellarp_1_1dns.md)**  |
| **[std](Namespaces/namespacestd.md)** <br>STL namespace.  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::dns::SRVData](Classes/classllarp_1_1dns_1_1SRVData.md)**  |
| struct | **[std::hash< llarp::dns::SRVData >](Classes/structstd_1_1hash_3_01llarp_1_1dns_1_1SRVData_01_4.md)**  |




## Source code

```cpp
#pragma once

#include "name.hpp"
#include "serialize.hpp"

#include <tuple>
#include <string_view>

#include "llarp/util/status.hpp"

namespace llarp::dns
{
  using SRVTuple = std::tuple<std::string, uint16_t, uint16_t, uint16_t, std::string>;

  class SRVData
  {
    static constexpr size_t TARGET_MAX_SIZE = 200;

    /* bind-like formatted string for SRV records in config file
     *
     * format:
     *   srv=service.proto priority weight port target
     *
     * exactly one space character between parts.
     *
     * if target is not empty, it must be either
     *  - simply a full stop (dot/period) OR
     *  - a name within the .loki or .snode subdomains. a target
     *    specified in this manner must not end with a full stop.
     *  - a full .loki or .snode address ending with a full stop.
     *  - the literal character '@' in which the primary flow layer address
     *    be it .loki or .snode is used.
     *
     * if empty target is provided the trailing space after port MUST be omitted.
     * when the target is empty it is treated like '@'.
     */
    void
    from_string(std::string_view srvString);

    [[nodiscard]] constexpr auto
    tuple_ref()
    {
      return std::tie(service_proto, priority, weight, port, target);
    }

   public:
    explicit SRVData(SRVTuple tup);
    explicit SRVData(std::string_view str);

    SRVData() = default;
    SRVData(const SRVData&) = default;
    SRVData(SRVData&&) = default;
    SRVData&
    operator=(SRVData&&) = default;
    SRVData&
    operator=(const SRVData&) = default;

    std::string service_proto;  // service and protocol may as well be together

    uint16_t priority;
    uint16_t weight;
    uint16_t port;

    // target string for the SRV record to point to
    // options:
    //   empty                     - dns query's qname
    //   @                         - dns query's qname
    //   dot                       - authoritative "no such service available"
    //   any other .loki or .snode - target is that .loki or .snode
    std::string target;

    // do some basic validation on the target string
    // note: this is not a conclusive, regex solution,
    // but rather some sanity/safety checks
    bool
    valid() const;

    [[nodiscard]] SRVTuple
    to_tuple() const;

    bool
    operator<(const SRVData& other) const;

    bool
    operator==(const SRVData& other) const;

    bool
    BEncode(llarp_buffer_t*) const;

    bool
    BDecode(llarp_buffer_t*);

    util::StatusObject
    ExtractStatus() const;
  };

}  // namespace llarp::dns

namespace std
{
  template <>
  struct hash<llarp::dns::SRVData>
  {
    size_t
    operator()(const llarp::dns::SRVData& data) const
    {
      const std::hash<std::string> h_str{};
      const std::hash<uint16_t> h_port{};
      return h_str(data.service_proto) ^ (h_str(data.target) << 3) ^ (h_port(data.priority) << 5)
          ^ (h_port(data.weight) << 7) ^ (h_port(data.port) << 9);
    }
  };
}  // namespace std
```


-------------------------------

Updated on 2023-03-09 at 21:57:14 +0000
